import java.util.*;

public class Block extends AttachableElement  {
   private static int id=0;  // Block identification number
   private final float mass;
   private Vector2D pos_t;     // current position at time t
   private Vector2D pos_tPlusDelta;  // next position in delta time in future
   private Vector2D speed_t;   // speed at time t
   private Vector2D speed_tPlusDelta;   // speed in delta time in future
   private Vector2D a_t;    // acceleration at time t
   private Vector2D a_tMinusDelta;  // acceleration delta time ago;
   private Vector<Spring> springs;  // Vector can grow, arrays cannot. 
	float roce = 0.5f;  // [N*s/m] 
   private Block(){   // nobody can create a block without state
     super(id++);
     mass=0;
   }
	//Inicializamos los atributos e instancias
   public Block(float mass, Vector2D position, Vector2D speed){
      super(id++);
      this.mass = mass;
		pos_t = position;
		pos_tPlusDelta = new Vector2D(0.0f, 0.0f);
		speed_t = speed;
		speed_tPlusDelta = new Vector2D(0.0f, 0.0f);
		a_t = new Vector2D(0.0f, 0.0f);
		a_tMinusDelta = new Vector2D(0.0f, 0.0f);
		springs = new Vector<Spring>();
   }
   public void attachSpring (Spring spring) {
     springs.add(spring);
   }
   public void detachSpring (Spring spring) {
     springs.remove(spring);
   }
   public Vector2D getPosition() {
      return pos_t;
   }
	//Sumamos todas las Fuerzas (incluida gravedad y roce)
   private Vector2D getNetForce(float g) {
     Vector2D force = new Vector2D(0.0f, 0.0f);
	  Vector2D fRoce = new Vector2D(-roce*speed_t.getX(),-roce*speed_t.getY());
     for (Spring s: springs){
       force.sumaVectorial(force, s.getForce(this));
	  }
     Vector2D auxFgravedad = new Vector2D(0.0f, (-mass*g));
	  force.sumaVectorial(force,auxFgravedad); //La gravedad solo resta el eje vertical Y
	  force.sumaVectorial(force,fRoce); //Roce proporcional a la velocidad
     return(force);    
   }
	
	public void setInitialState(float g) {
      a_tMinusDelta.setCoordenadas(getNetForce(g).getX()/mass,getNetForce(g).getY()/mass);
   }
   public void computeNextState(double delta_t, float g) {
     //Original: a_t = getNetForce(g)/mass;
	  a_t.setCoordenadas(getNetForce(g).getX()/mass,getNetForce(g).getY()/mass);
	  //Velocidad
     speed_tPlusDelta.setCoordenadas(speed_t.getX() + (3*a_t.getX() - a_tMinusDelta.getX())*delta_t/2, speed_t.getY() + (3*a_t.getY() - a_tMinusDelta.getY())*delta_t/2);
	  //Distancia
     pos_tPlusDelta.setCoordenadas(pos_t.getX() + speed_t.getX()*delta_t + (4*a_t.getX()-a_tMinusDelta.getX())*delta_t*delta_t/6, pos_t.getY() + speed_t.getY()*delta_t + (4*a_t.getY()-a_tMinusDelta.getY())*delta_t*delta_t/6);
	  
   }
	public void updateState(){
     pos_t.setCoordenadas(pos_tPlusDelta.getX(),pos_tPlusDelta.getY());
     speed_t.setCoordenadas(speed_tPlusDelta.getX(),speed_tPlusDelta.getY());
     //Original: a_tMinusDelta = a_t;
	  a_tMinusDelta.setCoordenadas(a_t.getX(),a_t.getY());
   }
	public String getDescription() {
      return "Block #" + getId()+": x,y";
   }
   public String getState() {
      return (getPosition().getX()+","+getPosition().getY()+"");
   }
}
