public class Spring extends PhysicsElement  {
   private static int id=0;  // Spring identification
   protected float restLength;
   private final float stiffness;
   protected AttachableElement a_end, b_end;

   private Spring(){   // nobody can create a block without state
      super(id++);
      stiffness=0;
   }
   public Spring(float restLength, float stiffness){
      super(id++);
      this.restLength = restLength;
      this.stiffness = stiffness;
      a_end = b_end = null;
   }
   public void attachAEnd (AttachableElement ae) {
      a_end = ae;
      ae.attachSpring(this);
   }
   public void detachAEnd(){
      if (a_end == null) return;
      a_end.detachSpring(this);
      a_end=null;
   }
   public void attachBEnd (AttachableElement ae) {
      b_end = ae;
      ae.attachSpring(this);
   }
   public void detachBEnd(){
      if (b_end == null) return;
      b_end.detachSpring(this);
      b_end = null;
   }
	
	
	//Hacia arriba nada modificado
   private double getLength() {
      if (a_end== null || b_end==null) return restLength;
      else 
         //Original: return a_end.getPosition()-b_end.getPosition();
			return Vector2D.distanciaVectorial(a_end.getPosition(),b_end.getPosition());
   }	
   public Vector2D getForce(AttachableElement ae) {
      //Original: double force = 0;
		Vector2D force = new Vector2D(0.0f, 0.0f);
		Vector2D delta_pos = new Vector2D(0.0f, 0.0f);
		if ((a_end == null) || (b_end == null))
         return force; //Original return 0;
      if ((ae != a_end) && (ae != b_end))
         return force; //Original return 0;		
     if (ae == a_end && b_end != null) { //Se representa el resorte como un vector de modulo restlength
 			double angulo = Vector2D.anguloVectorial(a_end.getPosition(),b_end.getPosition()); 
			force.setCoordenadas(Math.cos(angulo)*restLength,Math.sin(angulo)*restLength);//descomponemos Largo en reposo
         delta_pos.restaVectorial(a_end.getPosition(),b_end.getPosition());//diferencia en cartesianas
         force.restaVectorial(force,delta_pos);//Guardamos en force el stretch			
     }else
     if (ae == b_end && a_end != null) {	  
	   	double angulo = Vector2D.anguloVectorial(b_end.getPosition(),a_end.getPosition()); 
			force.setCoordenadas(Math.cos(angulo)*restLength,Math.sin(angulo)*restLength);//descomponemos Largo en reposo
         delta_pos.restaVectorial(b_end.getPosition(),a_end.getPosition());//diferencia en cartesianas
     		force.restaVectorial(force,delta_pos);//Guardamos en force el stretch
     }else {	  
       force = new Vector2D(0.0f, 0.0f);
     }
	  force.setCoordenadas(stiffness*force.getX(),stiffness*force.getY());//FuerzaResorte
     return force;
   }
	
	
   public String getDescription() {
      return "Spring #"+ getId()+": a_end , b_end ";
   }	
   public String getState() {
      String s;
      if (a_end != null) 
         //Original: s = a_end.getPosition()+",";
			s = a_end.getState()+",";
      else if (b_end != null)
              //Original: s = (b_end.getPosition()-restLength) +",";
				  s = Vector2D.largoResorte(b_end.getPosition(),-restLength)+",";
            else  
              s = 0.0+",";  // up to here left end is determined 
      if (b_end != null)
         //Original: s += b_end.getPosition();
			s += b_end.getState();
      else if (a_end != null)
               //Original: s += (a_end.getPosition()+restLength);
					s += Vector2D.largoResorte(a_end.getPosition(),restLength);
            else
               s += restLength;
      return s;
   }
}
